@@ -1,12 +1,16 @@
 use std::sync::Arc;
 
-use aho_corasick::{AhoCorasick, AhoCorasickBuilder};
+use aho_corasick::AhoCorasick;
 use dashmap::DashMap;
 use grep_searcher::LineStep;
 use log::Level::Trace;
 use once_cell::sync::Lazy;
-use regex::bytes::Regex;
 
+use super::embedding::{
+    RegexCache, RegexFamily, ENDING_LF_BLOCK_REGEX, ENDING_MARKDOWN_REGEX, END_SCRIPT, END_STYLE,
+    END_TEMPLATE,
+};
+use crate::LanguageType::LinguaFranca;
 use crate::{stats::CodeStats, utils::ext::SliceExt, Config, LanguageType};
 
 /// Tracks the syntax of the language as well as the current state in the file.
@@ -27,6 +31,7 @@ pub(crate) struct SyntaxCounter {
     pub(crate) quote_is_doc_quote: bool,
     pub(crate) stack: Vec<&'static str>,
     pub(crate) quote_is_verbatim: bool,
+    pub(crate) lf_embedded_language: Option<LanguageType>,
 }
 
 #[derive(Clone, Debug)]
@@ -51,6 +56,7 @@ pub(crate) enum LanguageContext {
     Html {
         language: LanguageType,
     },
+    LinguaFranca,
     Markdown {
         balanced: bool,
         language: LanguageType,
@@ -63,7 +69,8 @@ pub(crate) struct SharedMatchers {
     pub language: LanguageType,
     pub allows_nested: bool,
     pub doc_quotes: &'static [(&'static str, &'static str)],
-    pub important_syntax: AhoCorasick<u16>,
+    pub important_syntax: AhoCorasick,
+    #[allow(dead_code)]
     pub any_comments: &'static [&'static str],
     pub is_fortran: bool,
     pub is_literate: bool,
@@ -87,14 +94,14 @@ impl SharedMatchers {
     }
 
     pub fn init(language: LanguageType) -> Self {
-        fn init_corasick(pattern: &[&'static str], anchored: bool) -> AhoCorasick<u16> {
-            let mut builder = AhoCorasickBuilder::new();
-            builder
-                .anchored(anchored)
-                .byte_classes(false)
-                .dfa(true)
-                .prefilter(true);
-            builder.build_with_size(pattern).unwrap()
+        fn init_corasick(pattern: &[&'static str]) -> AhoCorasick {
+            AhoCorasick::builder()
+                .match_kind(aho_corasick::MatchKind::LeftmostLongest)
+                .start_kind(aho_corasick::StartKind::Unanchored)
+                .prefilter(true)
+                .kind(Some(aho_corasick::AhoCorasickKind::DFA))
+                .build(pattern)
+                .unwrap()
         }
 
         Self {
@@ -103,7 +110,7 @@ impl SharedMatchers {
             doc_quotes: language.doc_quotes(),
             is_fortran: language.is_fortran(),
             is_literate: language.is_literate(),
-            important_syntax: init_corasick(language.important_syntax(), false),
+            important_syntax: init_corasick(language.important_syntax()),
             any_comments: language.any_comments(),
             line_comments: language.line_comments(),
             multi_line_comments: language.multi_line_comments(),
@@ -130,6 +137,7 @@ impl SyntaxCounter {
             quote_is_doc_quote: false,
             quote_is_verbatim: false,
             stack: Vec::with_capacity(1),
+            lf_embedded_language: None,
             quote: None,
         }
     }
@@ -149,6 +157,12 @@ impl SyntaxCounter {
         !self.stack.is_empty()
     }
 
+    pub(crate) fn get_lf_target_language(&self) -> LanguageType {
+        // in case the target declaration was not found, default it to that language
+        const DEFAULT_LANG: LanguageType = LinguaFranca;
+        self.lf_embedded_language.unwrap_or(DEFAULT_LANG)
+    }
+
     #[inline]
     pub(crate) fn parse_line_comment(&self, window: &[u8]) -> bool {
         if self.quote.is_some() || !self.stack.is_empty() {
@@ -167,39 +181,39 @@ impl SyntaxCounter {
     }
 
     /// Try to see if we can determine what a line is from examining the whole
-    /// line at once. Returns `true` if sucessful.
-    pub(crate) fn can_perform_single_line_analysis(
+    /// line at once. Returns `true` if successful.
+    pub(crate) fn try_perform_single_line_analysis(
         &self,
         line: &[u8],
         stats: &mut crate::stats::CodeStats,
     ) -> bool {
-        if self.is_plain_mode() {
-            if line.trim().is_empty() {
-                stats.blanks += 1;
-                trace!("Blank No.{}", stats.blanks);
-                return true;
-            } else if !self.shared.important_syntax.is_match(line) {
-                trace!("^ Skippable");
+        if !self.is_plain_mode() {
+            false
+        } else if line.trim().is_empty() {
+            stats.blanks += 1;
+            trace!("Blank No.{}", stats.blanks);
+            true
+        } else if self.shared.important_syntax.is_match(line) {
+            false
+        } else {
+            trace!("^ Skippable");
 
-                if self.shared.is_literate
-                    || self
-                        .shared
-                        .line_comments
-                        .iter()
-                        .any(|c| line.starts_with(c.as_bytes()))
-                {
-                    stats.comments += 1;
-                    trace!("Comment No.{}", stats.comments);
-                } else {
-                    stats.code += 1;
-                    trace!("Code No.{}", stats.code);
-                }
-
-                return true;
+            if self.shared.is_literate
+                || self
+                    .shared
+                    .line_comments
+                    .iter()
+                    .any(|c| line.starts_with(c.as_bytes()))
+            {
+                stats.comments += 1;
+                trace!("Comment No.{}", stats.comments);
+            } else {
+                stats.code += 1;
+                trace!("Code No.{}", stats.code);
             }
-        }
 
-        false
+            true
+        }
     }
 
     pub(crate) fn perform_multi_line_analysis(
@@ -217,6 +231,8 @@ impl SyntaxCounter {
             }};
         }
 
+        let regex_cache = RegexCache::build(self.shared.language, lines, start, end);
+
         for i in start..end {
             if skip != 0 {
                 skip -= 1;
@@ -242,7 +258,7 @@ impl SyntaxCounter {
                 continue;
             }
 
-            if let Some(child) = self.parse_context(lines, i, end, config) {
+            if let Some(child) = self.parse_context(lines, i, end, config, &regex_cache) {
                 return AnalysisReport::ChildLanguage(child);
             }
 
@@ -348,6 +364,7 @@ impl SyntaxCounter {
         start: usize,
         end: usize,
         config: &Config,
+        regex_cache: &RegexCache,
     ) -> Option<FileContext> {
         use std::str::FromStr;
 
@@ -356,43 +373,35 @@ impl SyntaxCounter {
             return None;
         }
 
-        match self.shared.language {
-            LanguageType::Markdown | LanguageType::UnrealDeveloperMarkdown => {
-                static STARTING_MARKDOWN_REGEX: Lazy<Regex> =
-                    Lazy::new(|| Regex::new(r#"^```\S+\s"#).unwrap());
-                static ENDING_MARKDOWN_REGEX: Lazy<Regex> =
-                    Lazy::new(|| Regex::new(r#"```\s?"#).unwrap());
-
+        match regex_cache.family()? {
+            RegexFamily::Markdown(md) => {
                 if !lines[start..end].contains_slice(b"```") {
                     return None;
                 }
 
-                let opening_fence = STARTING_MARKDOWN_REGEX.find(&lines[start..end])?;
-                let start_of_code = start + opening_fence.end();
+                let opening_fence = md.starts_in_range(start, end)?;
+                let start_of_code = opening_fence.end();
                 let closing_fence = ENDING_MARKDOWN_REGEX.find(&lines[start_of_code..]);
                 if let Some(m) = &closing_fence {
-                    trace!("{:?}", String::from_utf8_lossy(m.as_bytes()))
+                    trace!("{:?}", String::from_utf8_lossy(m.as_bytes()));
                 }
                 let end_of_code = closing_fence
-                    .map(|fence| start_of_code + fence.start())
-                    .unwrap_or_else(|| lines.len());
-                let end_of_code_block = closing_fence
-                    .map(|fence| start_of_code + fence.end())
-                    .unwrap_or_else(|| lines.len());
+                    .map_or_else(|| lines.len(), |fence| start_of_code + fence.start());
+                let end_of_code_block =
+                    closing_fence.map_or_else(|| lines.len(), |fence| start_of_code + fence.end());
                 let balanced = closing_fence.is_some();
                 let identifier = &opening_fence.as_bytes().trim()[3..];
 
                 let language = identifier
                     .split(|&b| b == b',')
-                    .filter_map(|s| LanguageType::from_str(&String::from_utf8_lossy(s)).ok())
-                    .next()?;
+                    .find_map(|s| LanguageType::from_str(&String::from_utf8_lossy(s)).ok())?;
                 trace!(
                     "{} BLOCK: {:?}",
                     language,
                     String::from_utf8_lossy(&lines[start_of_code..end_of_code])
                 );
                 let stats =
-                    language.parse_from_slice(&lines[start_of_code..end_of_code].trim(), config);
+                    language.parse_from_slice(lines[start_of_code..end_of_code].trim(), config);
 
                 Some(FileContext::new(
                     LanguageContext::Markdown { balanced, language },
@@ -400,7 +409,7 @@ impl SyntaxCounter {
                     stats,
                 ))
             }
-            LanguageType::Rust => {
+            RegexFamily::Rust => {
                 let rest = &lines[start..];
                 let comment_syntax = if rest.trim_start().starts_with(b"///") {
                     b"///"
@@ -436,28 +445,34 @@ impl SyntaxCounter {
                     doc_block,
                 ))
             }
-            #[allow(clippy::trivial_regex)]
-            LanguageType::Html
-            | LanguageType::RubyHtml
-            | LanguageType::Svelte
-            | LanguageType::Vue => {
-                static START_SCRIPT: Lazy<Regex> =
-                    Lazy::new(|| Regex::new(r#"^<script(?:.*type="(.*)")?.*?>"#).unwrap());
-                static END_SCRIPT: Lazy<Regex> = Lazy::new(|| Regex::new(r#"</script>"#).unwrap());
-                static START_STYLE: Lazy<Regex> =
-                    Lazy::new(|| Regex::new(r#"^<style(?:.*lang="(.*)")?.*?>"#).unwrap());
-                static END_STYLE: Lazy<Regex> = Lazy::new(|| Regex::new(r#"</style>"#).unwrap());
-                static START_TEMPLATE: Lazy<Regex> =
-                    Lazy::new(|| Regex::new(r#"^<template(?:.*lang="(.*)")?.*?>"#).unwrap());
-                static END_TEMPLATE: Lazy<Regex> =
-                    Lazy::new(|| Regex::new(r#"</template>"#).unwrap());
+            RegexFamily::LinguaFranca(lf) => {
+                let opening_fence = lf.starts_in_range(start, end)?;
+                let start_of_code = opening_fence.end();
+                let closing_fence = ENDING_LF_BLOCK_REGEX.find(&lines[start_of_code..]);
+                let end_of_code = closing_fence
+                    .map_or_else(|| lines.len(), |fence| start_of_code + fence.start());
 
-                if let Some(captures) = START_SCRIPT.captures(&lines[start..end]) {
-                    let start_of_code = start + captures.get(0).unwrap().end();
+                let block_contents = &lines[start_of_code..end_of_code];
+                trace!("LF block: {:?}", String::from_utf8_lossy(block_contents));
+                let stats = self.get_lf_target_language().parse_from_slice(
+                    block_contents.trim_first_and_last_line_of_whitespace(),
+                    config,
+                );
+                trace!("-> stats: {:?}", stats);
+
+                Some(FileContext::new(
+                    LanguageContext::LinguaFranca,
+                    end_of_code,
+                    stats,
+                ))
+            }
+            RegexFamily::HtmlLike(html) => {
+                if let Some(mut captures) = html.start_script_in_range(start, end) {
+                    let start_of_code = captures.next().unwrap().end();
                     let closing_tag = END_SCRIPT.find(&lines[start_of_code..])?;
                     let end_of_code = start_of_code + closing_tag.start();
                     let language = captures
-                        .get(1)
+                        .next()
                         .and_then(|m| {
                             LanguageType::from_mime(&String::from_utf8_lossy(m.as_bytes().trim()))
                         })
@@ -476,12 +491,12 @@ impl SyntaxCounter {
                         end_of_code,
                         stats,
                     ))
-                } else if let Some(captures) = START_STYLE.captures(&lines[start..end]) {
-                    let start_of_code = start + captures.get(0).unwrap().end();
+                } else if let Some(mut captures) = html.start_style_in_range(start, end) {
+                    let start_of_code = captures.next().unwrap().end();
                     let closing_tag = END_STYLE.find(&lines[start_of_code..])?;
                     let end_of_code = start_of_code + closing_tag.start();
                     let language = captures
-                        .get(1)
+                        .next()
                         .and_then(|m| {
                             LanguageType::from_str(
                                 &String::from_utf8_lossy(m.as_bytes().trim()).to_lowercase(),
@@ -503,12 +518,12 @@ impl SyntaxCounter {
                         end_of_code,
                         stats,
                     ))
-                } else if let Some(captures) = START_TEMPLATE.captures(&lines[start..end]) {
-                    let start_of_code = start + captures.get(0).unwrap().end();
+                } else if let Some(mut captures) = html.start_template_in_range(start, end) {
+                    let start_of_code = captures.next().unwrap().end();
                     let closing_tag = END_TEMPLATE.find(&lines[start_of_code..])?;
                     let end_of_code = start_of_code + closing_tag.start();
                     let language = captures
-                        .get(1)
+                        .next()
                         .and_then(|m| {
                             LanguageType::from_str(
                                 &String::from_utf8_lossy(m.as_bytes().trim()).to_lowercase(),
@@ -534,7 +549,6 @@ impl SyntaxCounter {
                     None
                 }
             }
-            _ => None,
         }
     }
 
@@ -588,10 +602,13 @@ impl SyntaxCounter {
 
     #[inline]
     pub(crate) fn parse_end_of_quote(&mut self, window: &[u8]) -> Option<usize> {
+        #[allow(clippy::if_same_then_else)]
         if self._is_string_mode() && window.starts_with(self.quote?.as_bytes()) {
             let quote = self.quote.take().unwrap();
             trace!("End {:?}", quote);
             Some(quote.len())
+        } else if !self.quote_is_verbatim && window.starts_with(br"\\") {
+            Some(2)
         } else if !self.quote_is_verbatim
             && window.starts_with(br"\")
             && self
